#include <stddef.h>
#include <errno.h>

#include "termbps.h"

struct speedbps {
  speed_t speed;
  bps_t bps;
};

#include "termmac.h"

#if TERMMAC_TRIVIAL
# define tospeed(x)   (x)
# define fromspeed(x) (x)
#else

#define NSPEEDS ((sizeof speedbps)/(sizeof speedbps[0]))

static speed_t tospeed(bps_t b)
{
  size_t i;

  for (i = 0; i < NSPEEDS; i++)
    if (b == speedbps[i].bps)
      return speedbps[i].speed;

  errno = EINVAL;
  return (speed_t)-1;
}

static bps_t fromspeed(speed_t s)
{
  size_t i;

  for (i = 0; i < NSPEEDS; i++)
    if (s == speedbps[i].speed)
      return speedbps[i].bps;

  /* Don't clobber errno on an error return */
  if (s != (speed_t)-1)
    errno = EINVAL;

  return (bps_t)-1;
}

#endif	/* TERMMAC_TRIVIAL */

bps_t cfgetispeedbps(const struct termios *tio)
{
  return tospeed(cfgetispeed(tio));
}

bps_t cfgetospeedbps(const struct termios *tio)
{
  return tospeed(cfgetospeed(tio));
}

int cfsetispeedbps(struct termios *tio, bps_t x)
{
  speed_t b = tospeed(x);
  
  if (b == (speed_t)-1)
    return -1;
  else
    return cfsetispeed(tio, b);
}

int cfsetospeedbps(struct termios *tio, bps_t x)
{
  speed_t b = tospeed(x);
  
  if (b == (speed_t)-1)
    return -1;
  else
    return cfsetospeed(tio, b);
}
